#!/usr/bin/perl 
#
# recode_ped.pl : recodes initial pedigree file
# use "recode_ped.pl --help" or "man recode_ped.pl" for details
#-------------------------- help message begins -----------------

$name="recode_ped.pl";
$version="0.0.3";
$sep=",";
$out=",";
$na="";
$male="1";
$oped="pedigree.int.csv";
$CRIT_LEN=2*1024;
$TERM_MES="TOO MANY ERRORS, EXECUTION TERMINATED, NO FURTHER ERROR CHECK";
$FATAL_MES="FATAL ERROR, EXECUTION TERMINATED, NO FURTHER ERROR CHECK";

$help_message="
SRTNAM: RECODE_PED.PL 1

HEADER:
$name $version

NAME:
$name - A program for recoding pedigrees from free format to standard format

SYNOPSIS:
$name [--option] input_pedigree_file

Control the input format options:

[--help] [--sep INPUTSEPARATOR] [--na NA_CODE] 
[--male MALE_CODE][--out OUTPUTSEPARATOR] 

Control the output options:

[--oped output_pedigree_file]

DESCRIPTION:
Recodes pedigrees from free format to a more strict format.

INPUT FILE
Input_pedigree_file as a free format pedigree file. This is text file, where 
entries are assumed to be comma-separated and missing data indicated by empty 
entries (this may be changed using -s and -n options). The input file 
must contain a header line, providing the description of every column. 
Four or five columns are required with names id, fa, mo and sex. These 
contain a uniquie person identification number, unique paternal and 
maternal numbers and person's sex ($male = male by default, this 
can be changed with --male option). Also, pedigree identifier (ped) may be 
used. In this case, the id must be unique within the pedigree. 
The oder of columns is arbitrary, the additional columns may be included in 
the input file. In the pedigree file before a header line, there might be 
comment lines, starting with \"#\".

PROGRAM BLOCKS
The program will check most common pedigree errors 
(duplicated ids, male acting as mother, female acting as father, one parent 
missing, parents missing in id column, cycles). If errors are found, program 
will terminate with an error message. The programm will also split the data 
to disjoint pedigrees. The split will be compared with existing pedigree 
split, if \"ped\" identifier is provided, a possible error message will 
be generated if these two are different.
	STEP 0
Check the existence of input pedigree file. If fole does not exist FATAL ERROR 0.
	STEP 1
Check the header format: every name {id, fa, mo, sex} is present only one time;
name {ped} may be absent or present no more then one time. If the header format 
is incorrect FATAL ERROR 1. If name {ped} does not exist in the header line 
WARNING 1.
	STEP 2
Check the data_line's length. The column's numbers for id, fa, mo, sex and 
per (if exists) are difined. IMAX is maximal value of them. The data_line 
must contain no less then IMAX records. If it isn't true FATAL ERROR 2.
If length of line differs on length of the header line WARNING 2.
	STEP 3
Check the duplication of IDs in data_lines. If two or more lines contain 
the identical (ped+id) FATAL ERROR 3.
	STEP 4
a) Check the existance of $male symbol  and only two sex symbols in the file.
If it is not true ERROR 4. 
b) Check if all fathers have $male sex symbol and all mothers have $female 
sex symbol. If it is not true ERROR 4.
	STEP 5
Check  half-missing parents. If only one of them has ID, new virtual name
is introduced and new line are added to data file; WARNING 5.
	STEP 6
Check if all parents are in the ID column. If it is not true, the additional 
line with parent's ID is introduced in data file; WARNING 6.
	STEP 7
Extraction of connected structures from pedigree data and compare with original
pedigrees. If an original pedigree consists of more then one connected 
structures, WARNING 7.
	STEP 8
Check pedigree cycles. If person exists in the list of his/her ancestors, ERROR 8.
	STEP 9
Recoding of pedigree data and print in output file.

OUTPUT FILE
The program will generate files named $oped (the name may be changed 
with --oped option). The $oped file will contain a reformatted pedigree,
with ped, id, fa, mo, sex, orig_ped + orig_id columns and all other columns 
of initial filr. In this file,ids are coded as integers from 1 to total number 
of people in the sample + additional virtual persons. The parents will come 
before offspring. The sexes are coded as '1' for male and '2' for female. 
orig_ped and orig_id will contain original pedigree and id. Separator is defined 
as=','. It may be changed using -o option.
OPTIONS:
--help runs help for the command

--sep SEPARATOR specifies a separator between entries for the 
input_pedigree_file. A comma is assumed by default, but one can specify any 
character (e.g. \"--sep ZZxZ\")

--out OUTPUT_SEPARATOR specifies a separator between entries for the 
output_pedigree_file. A comma is assumed by default, but one can specify any 
character (e.g. \"--out ZZxZ\")

--na NA_CODE specifies a symbol (sequence) used to denote non-available 
(missing) data in the input_pedigree_file. An empty entry (\"$na\") is assumed 
by default, but one can specify any alphanumeric sequence (e.g. \"--na NA\") 

--male MALE_CODE specifies the code used for MALE in \"sex\" column; the 
alternate symbol is assumed to be used for female. By default, it is assumed 
that the males are encoded with \"$male\".

--oped FILE_NAME the name of file to store re-formatted pedigree date, 
$oped by default

RESOURCES:

DIAGNOSTICS:

SEE ALSO:

EXAMPLE:
recode_ped.pl --help

recode_ped.pl pedigree.dat

recode_ped.pl --na NA --sep \| --oped temp.ped pedigree.dat

COPYRIGHT:
recode_ped.pl is Copyright (c) 2004 Erasmus MC Rotterdam and ICG Novosibirsk

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 2 of the License, or (at your opinion) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 

BUGS:

AUTHOR:
Yurii Aulchenko (i.aoultchenko\@erasmusmc.nl)

VERSION:
$version  2006

CREATED:
04.06.2004 14:30

LAST MODIFIED:
17.02.2006 11.35

LANGUAGE:
Perl v5.8.4

OS: 
XXX Linux vXX.XX
";
#-------------------------- help message ends -----------------

# get command line arguments 
#**************** check the comand line ************

die "run recode_ped.pl --help for details\n" if (@ARGV<1);

if ($ARGV[0] eq "--help") {
	print "${help_message}";
#shoud be replaced with soft end!
	die ""; 
}; 

#*************** reparametrization *************************
if (@ARGV==1) {
	$inped=$ARGV[0];
	&PRN_PAR;
} 
else {
	$narg = @ARGV;
	$inped=$ARGV[$narg-1];
	for ($i=0;$i<$narg-1;$i+=2) 
	{
		$sep=$ARGV[$i+1] if ($ARGV[$i] =~ /--sep/);
		$out=$ARGV[$i+1] if ($ARGV[$i] =~ /--out/);
		$na=$ARGV[$i+1] if ($ARGV[$i] =~ /--na/);
		$male=$ARGV[$i+1] if ($ARGV[$i] =~ /--male/);
		$oped=$ARGV[$i+1] if ($ARGV[$i] =~ /--oped/);
	}
		if ($out eq "\\t"){
			$out = "\t";
		}
		if ($sep eq "\\t"){
			$sep = "\t";
		}
	&PRN_PAR;
}
#*********** open ERROR and WARNING files****************
$f_err="${inped}.err";
open (ERR,">$f_err");
$f_warn="${inped}.warn";
open (WARN,">$f_warn");
$error_m="";
$warning_m="";

#************open and read the pedigree data******************

#		STEP0

#***********check that input file may be opened *************************

open(INP,"$inped") or die "ERROR 0: cannot open pedigree file $inped: $!\n";

#		STEP1

#*********check the header line ********************
#*************where is header line?*******************
$hea="# ";
$line=0;
while ($hea =~ /^#/) {
	$line++;
	$hea = <INP>;
}
chomp $hea;

print "STEP 1: header in line $line, processing...\n";

#@header = split/$sep/,$hea;
@header = split(/$sep/, $hea, -1 );

$enid=$enped=$enfa=$enmo=$ensex=0;

#**************count the number of required entries

for ($i=0;$i<@header;$i++) {
	$enid++ if ($header[$i] =~ /^id$/);
	$enped++ if ($header[$i] =~ /^ped$/);
	$enfa++ if ($header[$i] =~ /^fa$/);
	$enmo++ if ($header[$i] =~ /^mo$/);
	$ensex++ if ($header[$i] =~ /^sex$/);
}

$warning_m = "${warning_m}WARNING 1: Pedigree (ped) column not found in input file, only the internal split will be available\n" if ($enped==0);

$error_m = "${error_m}ERROR 1: Personal ID (id) column not found in input file\n" if ($enid==0);
$error_m = "${error_m}ERROR 1: Father ID (fa) column not found in input file\n" if ($enfa==0);
$error_m = "${error_m}ERROR 1: Mother ID (mo) column not found in input file\n" if ($enmo==0);
$error_m = "${error_m}ERROR 1: Sex (sex) column not found in input file\n" if ($ensex==0);

$error_m = "${error_m}ERROR 1: Personal ID (id) column found more then once in input file\n" if ($enid>1);
$error_m = "${error_m}ERROR 1: Pedigree (ped) column found more then once in input file\n" if ($enped>1);
$error_m = "${error_m}ERROR 1: Father ID (fa) column found more then once in input file\n" if ($enfa>1);
$error_m = "${error_m}ERROR 1: Mother ID (mo) column found more then once in input file\n" if ($enmo>1);
$error_m = "${error_m}ERROR 1: Sex (sex) column found more then once in input file\n" if ($ensex>1);

$no_err=0;
if ($enid==0 or $enfa==0 or $enmo==0 or $ensex==0 or $enped>1 or $enid>1 or $enfa>1 or $enmo>1 or $ensex>1){$no_err=1};
	if ($warning_m ne "") {print "$warning_m"};
if ($no_err !=0) {
	print (WARN "$warning_m");
	print (ERR "$error_m");
	close INP;
	die "$error_m\n$FATAL_MES\n";
};

#find the position of required entries

for ($i=0;$i<@header;$i++) {
	$enid=$i+1 if ($header[$i] =~ /^id$/);
	$enped=$i+1 if ($header[$i] =~ /^ped$/);
	$enfa=$i+1 if ($header[$i] =~ /^fa$/);
	$enmo=$i+1 if ($header[$i] =~ /^mo$/);
	$ensex=$i+1 if ($header[$i] =~ /^sex$/);
}

#find the position of maximal right entry

$imax=0;
$imax=$enped if ($imax<$enped);
$imax=$enid if ($imax<$enid);
$imax=$enfa if ($imax<$enfa);
$imax=$enmo if ($imax<$enmo);
$imax=$ensex if ($imax<$ensex);

#header of additional variables
$head_trait="";
$n_tr=0;
for ($i=0;$i<@header;$i++) {
	next if $i eq $enped-1 or $i eq $enid-1 or $i eq $enfa-1 or $i eq $enmo-1 or $i eq $ensex-1;
	if($head_trait eq ""){
		 $head_trait="${head_trait}$header[$i]";
	}
	else{
	$head_trait="${head_trait}${out}$header[$i]";
	}
	$n_tr++;
}
#Traits for virtual persons
$tr_virt="";
for ($i=0;$i<$n_tr;$i++) {
	$tr_virt="${tr_virt}${out}";
}

#		STEP2 and STEP3

#read in the pedigree, check the size of lines (no less than $imax), duplicated IDs ...
print "STEPS 2 and 3:reading pedigree data, checking number of entries, 
duplicated IDs...\n";
$warning2="";
$noent = @header;
$nids=0;
$no_err=0;
while (<INP>) {
	$line++;
	next if (/^#/);
	next if (/^ *$/);
	chomp;
	$_="$_${sep}1";
	@temp = split/$sep/;
	$tmp=@temp-1;
	$warning2="";
	
        if ($temp[$enfa-1] eq "") {$temp[$enfa-1]="0"};
	if ($temp[$enmo-1] eq "") {$temp[$enmo-1]="0"};

	$tr="";
	for ($i=0;$i<$tmp;$i++) {
		next if $i eq $enped-1 or $i eq $enid-1 or $i eq $enfa-1 or $i eq $enmo-1 or $i eq $ensex-1;
#		if ($tr eq ""){		
#			$tr="${tr}$temp[$i]";
#		}
#		else{
			$tr="${tr}${out}$temp[$i]";
#		}
}


	if ($tmp ne $noent) {
		if ($tmp>=$imax){
			$warning2 = "WARNING 2: length of line $line is $tmp (not equal to number of entries in header, $noent)\n" ;
			print "$warning2\n";
		};
		$error_m = "${error_m}ERROR 2: length of line $line is $tmp (less that number of last pedigree entries in header, $imax)\n" if ($tmp < $imax);
	};
	$no_err++  if ($tmp < $imax);
	
	
	if ($enped>0) {
		$ttt = "$temp[$enped-1]_$temp[$enid-1]";
	} else {
		$ttt = "$temp[$enid-1]";
	}
		$id[$nids++] = $ttt;
	$line{$ttt}="$line" if (!(exists $hid{$ttt}));
	if (exists $hid{$ttt}) {
		$lili{$ttt}="$lili{$ttt}, $line";
		$no_err++;
	};
		$hid{$ttt}=$nids;
	if ($enped>0){
		if ($temp[$enfa-1] ne "0") {
			$fa{$ttt} = "$temp[$enped-1]_$temp[$enfa-1]";
		}else{
			$fa{$ttt} = "$temp[$enfa-1]";
		}				
		if ($temp[$enmo-1] ne "0") {
			$mo{$ttt} = "$temp[$enped-1]_$temp[$enmo-1]";
		}else{
			$mo{$ttt} = "$temp[$enmo-1]";
		}
	} else {
		$fa{$ttt} = "$temp[$enfa-1]";
		$mo{$ttt} = "$temp[$enmo-1]";
	}
		$sex{$ttt} = $temp[$ensex-1];
		$trait{$ttt}=$tr;
	$warning_m="${warning_m}${warning2}";
	splice @temp,0,$tmp;
}
$error_m="${error_m}ERROR 2: empty pedigree file\n" if ($nids<1);
$no_err++ if ($nids<1);
foreach(keys %lili) {
	$error_m="${error_m}ERROR 3: Personal ID $_ duplicated in lines $line{$_}$lili{$_}\n"; 
	}

close INP;
undef %lili;

if ($no_err!=0) {
	print (WARN "$warning_m"); 
	print (ERR "$error_m");
	if (length($error_m)>$CRIT_LEN) {
		$MESS=$TERM_MES;
	} else {
		$MESS=$FATAL_MES;
	}
	die "$error_m\n$MESS\n"
};

#		STEP4 a

#check sex errors
#a) -- only 2 declarated symbols used
print "STEP 4a: checking sex errors-- only 2 declarated symbols used\n";
$female="";

$n_err=1;

foreach (values %sex) {
	if ($_ eq $male) {$n_err=0};
}
%reverse=reverse(%sex);
@keys=keys(%reverse);
print "@keys\n";
$nn=scalar(@keys);
	$error_m="${error_m}ERROR 4: sex symbol $male absent in file\n"if ($n_err !=0);
	$error_m="${error_m}ERROR 4: there are $nn but not two sex symbols\n"if ($nn>2);


foreach(keys %sex) {
	if ($sex{$_} ne $male and !$female) {
		$female = $sex{$_}; 
		print "Symbol used for female = \"$female\"\n";
	};
}
if ($n_err !=0 or $nn>2){
	print (WARN "$warning_m"); 
	print (ERR $error_m);
	if (length($error_m)>$CRIT_LEN){
		die "$error_m\n $TERM_MES\n" ;
	} else {
		die "$error_m\n $FATAL_MES\n";
	}	
};
#		STEP5

#checking for half-missing parents...;
print "STEP 5: checking for half-missing parents...\n";

$n_virt=0;
foreach (keys %hid) {
	if ($enped>0){
		@temp=split/_/;
	}
	$warning5="";
	if ($fa{$_} eq "0" and $mo{$_} ne "0") {
		$warning5="WARNING 5: half-missing parents for id $_ (fa=$fa{$_}, mo=$mo{$_}), new fa_name and fa_line are added\n" ;	
		$n_virt++;
		if ($enped>0){
			$ttt="$temp[0]_virtual_$n_virt";
		}else{
			$ttt="virtual_$n_virt";
		}
		$fa{$_}=$ttt;
		$id{$ttt}=$ttt;
		$fa{$ttt}="0";
		$mo{$ttt}="0";
		$sex{$ttt}=$male;
		$trait{$ttt}=$tr_virt;
		$nids++;
		$hid{$ttt}=$nids;
	};
	if ($fa{$_} ne "0" and $mo{$_} eq "0") {
		$warning5="WARNING 5: half-missing parents for id $_ (fa=$fa{$_}, mo=$mo{$_}), new mo_name and mo_line are added\n" ;	
		$n_virt++;
		if ($enped>0){
			$ttt="$temp[0]_virtual_$n_virt";
		}else{
			$ttt="virtual_$n_virt";
		}
		$nids++;
		$hid{$ttt}=$nids;
		$mo{$_}=$ttt;
		$id{$ttt}=$ttt;
		$fa{$ttt}="0";
		$mo{$ttt}="0";
		$sex{$ttt}=$female;
		$trait{$ttt}=$tr_virt;
	};
	if ($warning5 ne ""){
	print "$warning5";
	$warning_m="${warning_m}${warning5}";
	};
}

#		STEP6

#check that all parents are persent in id column
print "STEP 6:checking if all parents are present in id column...\n";
foreach (keys %fa) {
	$warning6="";
	$fan = $fa{$_};
	next if ($fan eq "0");

	if (!(exists $fa{$fan})){
		$warning6="WARNING 6: Father $fan of person $_ (line $line{$_}) is not presented in id column, new fa_line is added\n" ;	
		$ttt=$fan;
		$id{$ttt}=$ttt;
		$fa{$ttt}="0";
		$mo{$ttt}="0";
		$sex{$ttt}=$male;
		$trait{$ttt}=$tr_virt;
		$nids++;
		$hid{$ttt}=$nids;
		print "$warning6";
		$warning_m="${warning_m}${warning6}";
	};
}

foreach (keys %mo) {
	$warning6="";
	$fan = $mo{$_};
	next if ($fan eq "0");
	if (!(exists $mo{$fan})){
		$warning6="WARNING 6: Mother $fan of person $_ (line $line{$_}) is not presented in id column, new mo_line is added\n" ;	
		$ttt=$fan;
		$id{$ttt}=$ttt;
		$fa{$ttt}="0";
		$mo{$ttt}="0";
		$sex{$ttt}=$female;
		$trait{$ttt}=$tr_virt;
		$nids++;
		$hid{$ttt}=$nids;
		print "$warning6";
		$warning_m="${warning_m}${warning6}";
	};
}


#		STEP4 b


print "STEP 4b: checking if fathers are males and mothers are females...\n";
#check sex errors -- fathers are males
foreach(keys %fa) {
	next if ($fa{$_} eq "0");

	$error_m = "${error_m}ERROR 4: Father $fa{$_} of $_ (line $line{$_}) is not male (${sex{$fa{$_}}} ne $male)\n" if ($sex{$fa{$_}} ne $male);
}

if (length($error_m)>$CRIT_LEN) {
	print (WARN "$warning_m");
	print (ERR "$error_m");
	die "$error_m\n$TERM_MES\n"; 
};

#check sex errors -- mothers are females
foreach(keys %mo) {
	next if ($mo{$_} eq "0");
	$error_m = "${error_m}ERROR 4: Mother $mo{$_} of $_ (line $line{$_}) is not female (${sex{$mo{$_}}} ne $female)\n" if ($sex{$mo{$_}} ne $female);
}
if ($error_m ne ""){
	print (WARN "$warning_m");
	print (ERR "$error_m");
	if (length($error_m)>$CRIT_LEN) {
		die "$error_m\n$TERM_MES\n"; 
	}else {
		die "$error_m\n$FATAL_MES\n" ;
	}
};
#		STEP7

#identify the pedigrees and compare with original pedigrees; 
print "STEP 7: checking pedigree structure (connected parts)\n";
($sec,$min,$hour,$mday,$mon,$year)=localtime;
print "${hour}:${min}:${sec}\n";

$NP=0;
$i=0;
foreach (keys %hid) {
	$i++;
	print "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b $i";
	if ($n_ped{$_}) {
		if ($fa{$_} ne "0") {
			if (!(exists $n_ped{$fa{$_}})) {
				$n_ped{$fa{$_}}=$n_ped{$_};
			};
			if (!(exists $n_ped{$mo{$_}})) {
				$n_ped{$mo{$_}}=$n_ped{$_};
			};
		}
	} else {
	
		if ($fa{$_} ne "0") {
			if ($n_ped{$fa{$_}}) {
				$n_ped{$_}=$n_ped{$fa{$_}};
				$n_ped{$mo{$_}}=$n_ped{$_} if (!(exists $n_ped{$mo{$_}}));
			} else {
				if ($n_ped{$mo{$_}}) {
					$n_ped{$_}=$n_ped{$mo{$_}};
					$n_ped{$fa{$_}}=$n_ped{$mo{$_}};
				} else {
					$NP++;
					$n_ped{$_}=$NP;
					$n_ped{$fa{$_}}=$NP;
					$n_ped{$mo{$_}}=$NP;
					if ($enped>0){
						@temp=split/_/;
						$old=$temp[0];
					$lili{$NP}="$old";
					}else{
					$lili{$NP}="$NP";
					}
					
				}
			}
		}else{
			$NP++;
			$n_ped{$_}=$NP;
			if ($enped>0){
				@temp=split/_/;
				$old=$temp[0];
			$lili{$NP}="$old";
			}else{
			$lili{$NP}="$NP";
			}
		
		}
	}
# glucking the related pedigrees (renumeration)
		next if ($fa{$_} eq "0" or ($n_ped{$_} eq $n_ped{$fa{$_}} and $n_ped{$_} eq $n_ped{$mo{$_}}));
		$nfa=$n_ped{$fa{$_}};
		$nmo=$n_ped{$mo{$_}};
		$nn=$n_ped{$_};
		foreach (values %n_ped) {
			if ($_ eq $nfa) {$_=$nn};		
			if ($_ eq $nmo) {$_=$nn};		
			if ($nfa ne $nn) {delete ($lili{$nfa})};
			if ($nmo ne $nn) {delete ($lili{$nmo})};
		}
}	

#		STEP 8

# checking  cycles
print "\nSTEP 8: checking pedigree cycles\n";
($sec,$min,$hour,$mday,$mon,$year)=localtime;
print "${hour}:${min}:${sec}\n";
$iii=0;
foreach (keys %hid) {
	$iii++;
###	print "$iii\n";
	print "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b $iii";
	if ($fa{$_} ne "0"){
		$array[0]=$_;
		$test=$_;
		$n1=0;
		$n2=1;
		$n_err=0;
		&CYCLE;
		if ($n_err ne 0){
			$error_m="${error_m}ERROR 8: cycle around person $array[0]\n";
		}
		$nnn=scalar(@array);
		splice @array,1,$nnn;
	};	
}
if ($error_m ne ""){
	print (ERR $error_m);
	if ($warning_m ne ""){print (WARN $warning_m)};
	if (length($error_m)>$CRIT_LEN) {
		die "$error_m\n$TERM_MES\n"; 
	}else {
		die "$error_m\n$FATAL_MES\n" ;
	}
}

#		STEP 9
#recoding of pedigree data and print in output file
print "\nSTEP 9: recoding of pedigree data and print in output file\n";
($sec,$min,$hour,$mday,$mon,$year)=localtime;
print "${hour}:${min}:${sec}\n";

#	calculation of pedigree sizes

foreach (keys %lili){
	$nnn{$_}=0;
}
foreach (keys %hid){
	$nnn{$n_ped{$_}}++;
}

#	sorting of pedigrees from max_size to min
$nn=0;
$npp[0]=0;
$max=1;
M:
while ($max>0){
	$max=0;
	foreach (keys %lili){	
		next if ($nnn{$_} < $max);
		$n_p=$_;
		$max=$nnn{$_};
	}	
	if ($max>0){
		$nn++;
		$npp[$nn-1]=$n_p;
		$mimi{$nn}="$lili{$n_p}";
		delete ($lili{$n_p});
	} 
	goto M;
}
# checking that initial pedigree consist of more than one connected structures
if ($enped>0){
	
	foreach (keys %mimi){
		$K=$_;
		$VV=$mimi{$_};
		delete ($mimi{$_});
		$nn=0;
		for $k1 (keys %mimi){
			if ($mimi{$k1} eq $VV){
		 	        if ($nn eq 0){
				$warning_m="${warning_m}WARNING 7: initial pedigree  $mimi{$k1} has unconnected parts with new names $K, $k1\n";
				}else{
				chomp ($warning_m);
				$warning_m="${warning_m}, $k1\n";
				}
				$nn++;
				delete ($mimi{$k1});
			};
		}
	}
}

# recoding of sex
if ($male ne "1" or $female ne "2"){
	foreach (keys %hid){
		if ($sex{$_} eq $male) {
			$sex{$_}="1";
		}else{
			$sex{$_}="2";
		}

	}
}					
open (OPED,">$oped");

print (OPED "ped${out}id${out}fa${out}mo${out}sex${out}kod${out}${head_trait}\n");

$NK=0;
$kkk=0;
foreach $temp (@npp){
	$NK++;
	&WRITE;
}
close OPED;

if ($warning_m ne ""){
	 print (WARN "$warning_m");
}
print " The program is successfully finished\n";



sub PRN_PAR {
print "$name v. $version input patameters
\tField input_separator: --sep=\"$sep\"
\tField output_separator: --out=\"$out\"
\tNon-available coded as: --na=\"$na\"
\tMale coded as: --male=\"$male\"
\tInput pedigree file: \"$inped\"
\tOutput pedigree file: --oped=\"$oped\"
";
}

sub WRITE{
    $nnn=0;
    foreach (keys %hid){
	next if ($n_ped{$_} ne $temp);
	if ($fa{$_} eq "0"){
		$kkk++;
		print (OPED "$NK${out}$kkk${out}0${out}0${out}$sex{$_}${out}$_$trait{$_}\n");
		$kod{$_}=$kkk;
		delete ($hid{$_});
		$nnn++; 
	}
}
    foreach (keys %hid){
	next if ($n_ped{$_} ne $temp);
       	if (exists $kod{$fa{$_}} and exists $kod{$mo{$_}}){	
	 		$kkk++;
			print (OPED "$NK${out}$kkk${out}$kod{$fa{$_}}${out}$kod{$mo{$_}}${out}$sex{$_}${out}$_$trait{$_}\n");
			$kod{$_}=$kkk;
			delete ($hid{$_});
			$nnn++; 
		}
	}
   	if ($nnn >0){
	&WRITE;
    }
}


sub CYCLE {
	for ($j1=0;$j1<$n2;$j1++){
		$nnn=0;
		if ($array[$j1] eq $test){
			$nnn++;
		};
	}
	if ($nnn>1){
		$n_err++;
		$mer{$n_err}=$test;
	}

	if ($n_err eq 0) {
	$nn=$n1;
		for ($i=$n1; $i<$n2;$i++){
			$nn++;	
			if ($fa{$array[$i]} ne "0"){
				$fam=$fa{$array[$i]};
				$mom=$mo{$array[$i]};
				if ($fam eq $test or $mom eq $test) {$n_err++};
				$nf=0;
				$nm=0;
				for ($j=1; $j<$n2;$j++){
				   if  ($array[$j] eq $fam){
					$nf++;
				   }
				   if  ($array[$j] eq $mom){
					$nm++;
				   }
				}
			
			if ($nf eq 0) {push @array,"$fam"};
			if ($nm eq 0) {push @array,"$mom"};
	                }
		}
		$n1=$nn;
		$n2=scalar(@array);
		if ($n_err eq 0 and $n1 < $n2){
			&CYCLE;
		}
	}
}

